@@ -28,7 +28,6 @@ class PBOPool {
 public:
 	PBOPool(size_t pbo_size = 8 << 20,  // 8 MB, large enough for 1080p 4:2:2.
                 size_t num_pbos = 8,
-                GLenum buffer = GL_PIXEL_UNPACK_BUFFER_ARB,
                 GLenum permissions = GL_MAP_WRITE_BIT,
                 GLenum map_bits = GL_MAP_FLUSH_EXPLICIT_BIT);
 
@@ -42,7 +41,7 @@ private:
 	std::queue<PBO> freelist;
 
 	size_t pbo_size;
-	GLenum buffer, permissions, map_bits;
+	GLenum permissions, map_bits;
 };
 
 extern PBOPool *global_pbo_pool;
