@@ -4,6 +4,7 @@
 #include "clip_list.h"
 #include "frame_on_disk.h"
 #include "queue_spot_holder.h"
+#include "shared/metrics.h"
 
 extern "C" {
 #include <libavformat/avformat.h>
@@ -137,6 +138,7 @@ private:
 	std::atomic<int64_t> metric_refresh_frame{ 0 };
 	std::atomic<int64_t> metric_interpolated_frame{ 0 };
 	std::atomic<int64_t> metric_interpolated_faded_frame{ 0 };
+	Summary metric_player_ahead_seconds;
 
 	// under queue_state_mu. Part of this instead of VideoStream so that we own
 	// its lock and can sleep on it.
