@@ -849,13 +849,13 @@ void VideoStream::encode_thread_func()
 	}
 }
 
-int VideoStream::write_packet2_thunk(void *opaque, uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
+int VideoStream::write_packet2_thunk(void *opaque, FF_MAYBE_CONST uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
 {
 	VideoStream *video_stream = (VideoStream *)opaque;
 	return video_stream->write_packet2(buf, buf_size, type, time);
 }
 
-int VideoStream::write_packet2(uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
+int VideoStream::write_packet2(FF_MAYBE_CONST uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
 {
 	if (type == AVIO_DATA_MARKER_SYNC_POINT || type == AVIO_DATA_MARKER_BOUNDARY_POINT) {
 		seen_sync_markers = true;
