@@ -12,7 +12,7 @@ int main(int argc, char **argv)
 {
 	if (argc != 3) {
 		fprintf(stderr, "Usage: ./vis input.flo out.ppm\n");
-		exit(1);
+		abort();
 	}
 
 	Flow flow = read_flow(argv[1]);
