@@ -1,27 +1,36 @@
 #include "alsa_pool.h"
 
 #include <alsa/asoundlib.h>
+#include <alsa/control.h>
+#include <alsa/error.h>
+#include <alsa/pcm.h>
 #include <assert.h>
 #include <errno.h>
 #include <limits.h>
+#include <linux/limits.h>
+#include <mutex>
 #include <pthread.h>
 #include <poll.h>
 #include <stdint.h>
 #include <stdio.h>
+#include <string>
 #include <sys/eventfd.h>
 #include <sys/inotify.h>
+#include <sys/poll.h>
+#include <thread>
 #include <unistd.h>
 #include <algorithm>
 #include <chrono>
 #include <functional>
 #include <iterator>
 #include <memory>
-#include <ratio>
+#include <vector>
 
 #include "alsa_input.h"
 #include "audio_mixer.h"
-#include "defs.h"
+#include "card_type.h"
 #include "input_mapping.h"
+#include "shared/shared_defs.h"
 #include "state.pb.h"
 
 using namespace std;
