@@ -1,13 +1,32 @@
 #include "analyzer.h"
 
+#include <QComboBox>
+#include <QEvent>
 #include <QDialogButtonBox>
+#include <QImage>
+#include <QMainWindow>
 #include <QMouseEvent>
+#include <QPixmap>
+#include <QPushButton>
 #include <QPen>
+#include <QRgb>
 #include <QSurface>
 #include <QTimer>
 
+#include <algorithm>
+#include <functional>
 #include <movit/resource_pool.h>
 #include <movit/util.h>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include <assert.h>
+#include <math.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <string.h>
 
 #include "shared/context.h"
 #include "flags.h"
@@ -52,9 +71,9 @@ Analyzer::Analyzer()
 	signal_changed();
 	ui->grabbed_frame_label->installEventFilter(this);
 
-        glGenBuffers(1, &pbo);
-        glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, pbo);
-        glBufferData(GL_PIXEL_PACK_BUFFER_ARB, global_flags.width * global_flags.height * 4, nullptr, GL_STREAM_READ);
+	glGenBuffers(1, &pbo);
+	glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, pbo);
+	glBufferData(GL_PIXEL_PACK_BUFFER_ARB, global_flags.width * global_flags.height * 4, nullptr, GL_STREAM_READ);
 }
 
 Analyzer::~Analyzer()
@@ -219,7 +238,7 @@ bool Analyzer::eventFilter(QObject *watched, QEvent *event)
 		ui->blue_label->setText(u8"—");
 		ui->hex_label->setText(u8"#—");
 	}
-        return false;
+	return false;
 }
 
 void Analyzer::grab_pixel(int x, int y)
