@@ -3,26 +3,40 @@
 #include <assert.h>
 #include <bmusb/bmusb.h>
 #include <endian.h>
+#include <map>
 #include <math.h>
+#include <memory>
+#include <mutex>
+#include <set>
+#include <string>
+#include <vector>
 #ifdef __SSE2__
 #include <immintrin.h>
 #endif
+#include <math.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <algorithm>
 #include <chrono>
-#include <cmath>
-#include <cstddef>
 #include <limits>
 #include <utility>
 
+#include "alsa_pool.h"
+#include "card_type.h"
 #include "decibel.h"
+#include "defs.h"
+#include "filter.h"
 #include "flags.h"
+#include "input_mapping.h"
+#include "resampling_queue.h"
 #include "shared/metrics.h"
+#include "shared/shared_defs.h"
 #include "state.pb.h"
-#include "shared/timebase.h"
+#include "stereocompressor.h"
 
 using namespace bmusb;
 using namespace std;
