@@ -1,8 +1,11 @@
 #include "chroma_subsampler.h"
 #include "v210_converter.h"
 
+#include <string>
 #include <vector>
 
+#include <assert.h>
+#include <stdio.h>
 #include <movit/effect_util.h>
 #include <movit/resource_pool.h>
 #include <movit/util.h>
