@@ -1,10 +1,15 @@
+#include <QAction>
 #include <QActionGroup>
+#include <QKeySequence>
 #include <QMenu>
 #include <QObject>
 
+#include <bmusb/bmusb.h>
+#include <cstdint>
 #include <map>
 
 #include "mixer.h"
+#include "shared/shared_defs.h"
 
 using namespace std;
 
