@@ -1,19 +1,39 @@
-#include <movit/effect_util.h>
+#include "DeckLinkAPI.h"
+#include "DeckLinkAPIConfiguration.h"
+#include "DeckLinkAPIModes.h"
+#include "DeckLinkAPITypes.h"
+#include "LinuxCOM.h"
+#include <atomic>
+#include <assert.h>
+#include <chrono>
+#include <inttypes.h>
+#include <math.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <string.h>
+#include <memory>
+#include <movit/image_format.h>
 #include <movit/util.h>
 #include <movit/resource_pool.h>  // Must be above the Xlib includes.
 #include <pthread.h>
-#include <unistd.h>
+#include <thread>
 
 #include <mutex>
 
 #include <epoxy/egl.h>
+#include <utility>
+#include <vector>
 
 #include "chroma_subsampler.h"
 #include "decklink_output.h"
 #include "decklink_util.h"
 #include "flags.h"
+#include "ref_counted_frame.h"
 #include "shared/metrics.h"
 #include "print_latency.h"
+#include "shared/ref_counted_gl_sync.h"
+#include "shared/shared_defs.h"
 #include "shared/timebase.h"
 #include "v210_converter.h"
 
