@@ -526,7 +526,11 @@ AVPixelFormat get_hw_format(AVCodecContext *ctx, const AVPixelFormat *fmt)
 			if (config->pix_fmt == *fmt_ptr) {
 				fprintf(stderr, "Initialized '%s' hardware decoding for codec '%s'.\n",
 					av_hwdevice_get_type_name(type), ctx->codec->name);
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(60, 26, 100)
 				if (ctx->profile == FF_PROFILE_H264_BASELINE) {
+#else
+				if (ctx->profile == AV_PROFILE_H264_BASELINE) {
+#endif
 					fprintf(stderr, "WARNING: Stream claims to be H.264 Baseline, which is generally poorly supported in hardware decoders.\n");
 					fprintf(stderr, "         Consider encoding it as Constrained Baseline, Main or High instead.\n");
 					fprintf(stderr, "         Decoding might fail and fall back to software.\n");
