@@ -1,10 +1,12 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
+#include <memory>
 #include <stdbool.h>
 #include <sys/types.h>
 #include <QMainWindow>
 #include <QString>
+#include <QWidget>
 #include <chrono>
 #include <string>
 #include <vector>
@@ -13,6 +15,7 @@
 #include "audio_mixer.h"
 #include "midi_mapper.h"
 #include "mixer.h"
+#include "theme.h"
 
 class QDial;
 class QEvent;
