@@ -1,9 +1,11 @@
 #include <cef_app.h>
 #include <cef_browser.h>
-#include <cef_client.h>
-#include <cef_version.h>
+#include <cef_command_line.h>
+#include <cef_task.h>
 #include <QTimer>
 #include <QWidget>
+#include <mutex>
+#include <thread>
 
 #include "nageru_cef_app.h"
 
@@ -16,6 +18,7 @@ void NageruCefApp::OnBeforeCommandLineProcessing(
 	command_line->AppendSwitch("disable-gpu");
 	command_line->AppendSwitch("disable-gpu-compositing");
 	command_line->AppendSwitch("enable-begin-frame-scheduling");
+	command_line->AppendSwitchWithValue("autoplay-policy", "no-user-gesture-required");
 
 	// https://bitbucket.org/chromiumembedded/cef/issues/2717/xmlhttprequest-empty-responsetext
 	command_line->AppendSwitch("disable-web-security");
