@@ -1,18 +1,27 @@
-#include "video_encoder.h"
-
+#include <algorithm>
 #include <assert.h>
+#include <chrono>
+#include <epoxy/gl.h>
+#include <memory>
+#include <movit/image_format.h>
+#include <mutex>
+#include <netdb.h>
+#include <stdint.h>
 #include <stdio.h>
+#include <string.h>
+#include <string>
+#include <sys/socket.h>
+#include <thread>
 #include <time.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
-#include <string>
-#include <thread>
-#include <chrono>
+#include <vector>
 
 extern "C" {
 #include <libavutil/mem.h>
+#include <libavformat/avformat.h>
+#include <libavformat/avio.h>
+#include <libavutil/avutil.h>
+#include <libavutil/rational.h>
 }
 
 #include "audio_encoder.h"
@@ -27,6 +36,10 @@ extern "C" {
 #include "quicksync_encoder.h"
 #include "shared/timebase.h"
 #include "x264_encoder.h"
+#include "video_encoder.h"
+#include "shared/metrics.h"
+#include "shared/ref_counted_gl_sync.h"
+#include "shared/shared_defs.h"
 
 class RefCountedFrame;
 
@@ -277,13 +290,13 @@ void VideoEncoder::open_output_streams()
 	}
 }
 
-int VideoEncoder::write_packet2_thunk(void *opaque, uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
+int VideoEncoder::write_packet2_thunk(void *opaque, FF_MAYBE_CONST uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
 {
 	VideoEncoder *video_encoder = (VideoEncoder *)opaque;
 	return video_encoder->write_packet2(buf, buf_size, type, time);
 }
 
-int VideoEncoder::write_packet2(uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
+int VideoEncoder::write_packet2(FF_MAYBE_CONST uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time)
 {
 	if (type == AVIO_DATA_MARKER_SYNC_POINT || type == AVIO_DATA_MARKER_BOUNDARY_POINT) {
 		seen_sync_markers = true;
@@ -302,7 +315,7 @@ int VideoEncoder::write_packet2(uint8_t *buf, int buf_size, AVIODataMarkerType t
 	return buf_size;
 }
 
-int VideoEncoder::write_srt_packet_thunk(void *opaque, uint8_t *buf, int buf_size)
+int VideoEncoder::write_srt_packet_thunk(void *opaque, FF_MAYBE_CONST uint8_t *buf, int buf_size)
 {
 	VideoEncoder *video_encoder = (VideoEncoder *)opaque;
 	return video_encoder->write_srt_packet(buf, buf_size);
@@ -460,7 +473,7 @@ int VideoEncoder::connect_to_srt()
 	return -1;
 }
 
-int VideoEncoder::write_srt_packet(uint8_t *buf, int buf_size)
+int VideoEncoder::write_srt_packet(FF_MAYBE_CONST uint8_t *buf, int buf_size)
 {
 	if (want_srt_metric_update.exchange(false) && srt_sock != -1) {
 		srt_metrics.update_srt_stats(srt_sock);
