@@ -6,6 +6,7 @@
 #define _VIDEO_ENCODER_H
 
 #include <epoxy/gl.h>
+#include <epoxy/gl_generated.h>
 #include <movit/image_format.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -14,7 +15,6 @@
 #include <mutex>
 #include <string>
 #include <vector>
-#include <chrono>
 
 extern "C" {
 #include <libavformat/avformat.h>
@@ -23,6 +23,7 @@ extern "C" {
 
 #include <srt/srt.h>
 
+#include "shared/ff_maybe_const.h"
 #include "shared/mux.h"
 #include "shared/ref_counted_gl_sync.h"
 #include "srt_metrics.h"
@@ -80,11 +81,11 @@ public:
 
 private:
 	void open_output_streams();
-	static int write_packet2_thunk(void *opaque, uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time);
-	int write_packet2(uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time);
+	static int write_packet2_thunk(void *opaque, FF_MAYBE_CONST uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time);
+	int write_packet2(FF_MAYBE_CONST uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time);
 
-	static int write_srt_packet_thunk(void *opaque, uint8_t *buf, int buf_size);
-	int write_srt_packet(uint8_t *buf, int buf_size);
+	static int write_srt_packet_thunk(void *opaque, FF_MAYBE_CONST uint8_t *buf, int buf_size);
+	int write_srt_packet(FF_MAYBE_CONST uint8_t *buf, int buf_size);
 	int open_srt_socket();  // Returns -1 on error.
 	int connect_to_srt();  // Returns -1 on error.
 
