@@ -3,6 +3,8 @@
 
 #include <ratio>
 
+#include <stdint.h>
+
 // Common timebase that allows us to represent one frame exactly in all the
 // relevant frame rates:
 //
@@ -15,7 +17,7 @@
 // If we also wanted to represent one sample at 48000 Hz, we'd need
 // to go to 300000. Also supporting one sample at 44100 Hz would mean
 // going to 44100000; probably a bit excessive.
-#define TIMEBASE 120000
+constexpr int64_t TIMEBASE = 120000;
 
 // Some muxes, like MP4 (or at least avformat's implementation of it),
 // are not too fond of values above 2^31. At timebase 120000, that's only
